#!/usr/bin/env python
# RatBZEdit is a BZFlag map editor made with Python.
# Copyright (C) 2008  Clayton G. Hobbs
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#-----{ Imports / Initialization }--------------------------------------------

try:
    import os
    import shelve
    import time
    import wx
    import images
    from wx.lib.wordwrap import wordwrap
except ImportError, error:
    print 'Error importing module: %s' % error
    exit()

ID_Box = wx.NewId()
ID_Pyramid = wx.NewId()
ID_Base = wx.NewId()

#-----{ SplashScreen Class }--------------------------------------------------

class SplashScreen(wx.SplashScreen):
    def __init__(self, parent=None):
        splashImage = wx.Image(name = "images/splash.png").ConvertToBitmap()
        splashStyle = wx.SPLASH_CENTRE_ON_SCREEN | wx.SPLASH_TIMEOUT
        splashDuration = 2000
        wx.SplashScreen.__init__(self, splashImage, splashStyle,
                                 splashDuration, parent)
        
        self.Bind(wx.EVT_CLOSE, self.onExit)
        
        wx.Yield()
        
    def onExit(self, evt):
        self.Hide()
        win = MainWindow(None, -1)
        app.SetTopWindow(win)
        win.Show()
        evt.Skip()

#-----{ MainSplitter Class }--------------------------------------------------

class MainSplitter(wx.SplitterWindow):
    def __init__(self, parent, ID):
        wx.SplitterWindow.__init__(self, parent, ID, style=wx.SP_LIVE_UPDATE)

#-----{ StatusBar Class }-----------------------------------------------------

class StatusBar(wx.StatusBar):
    def __init__(self, parent):
        wx.StatusBar.__init__(self, parent)
        
        # Initialize the status bar.
        self.SetFieldsCount(2)
        self.SetStatusWidths([-9, -5])
        
        # Set status text for field 0
        self.SetStatusText('RatBZEdit Ready', 0)
        
        # Set status text for field 1
        self.timer = wx.PyTimer(self.clock)
        self.timer.Start(1000)
        self.clock()
    
    # Clock function
    def clock(self):
        t = time.localtime(time.time())
        st = time.strftime('%c', t)
        self.SetStatusText(st, 1)

#-----{ MainWindow Class }----------------------------------------------------

class MainWindow(wx.Frame):
    def __init__(self, parent, ID, title='RatBZEdit', pos=wx.DefaultPosition,
                 size=(700, 600), style=wx.DEFAULT_FRAME_STYLE):
        wx.Frame.__init__(self, parent, ID, title, pos, size, style)
        
        # Menu bar:
        menuBar = wx.MenuBar()
        
        # File menu:
        fileMenu = wx.Menu()
        fileMenu.Append(wx.ID_NEW, '&New\tCtrl-N', 'Create a new .bzw file')
        fileMenu.Append(wx.ID_OPEN, '&Open\tCtrl-O', 'Open a .bzw file' )
        fileMenu.AppendSeparator()
        fileMenu.Append(wx.ID_SAVE, '&Save\tCtrl-S',
                        'Save the current .bzw file')
        fileMenu.Append(wx.ID_SAVEAS, 'Save &As\tShift-Ctrl-S',
                        'Save the current .bzw file under a different name')
        fileMenu.AppendSeparator()
        fileMenu.Append(wx.ID_EXIT, '&Quit\tCtrl-Q', 'Quit RatBZEdit')
        # Add the file menu to the menu bar
        menuBar.Append(fileMenu, '&File')
        
        # Edit menu:
        editMenu = wx.Menu()
        editMenu.Append(wx.ID_UNDO, '&Undo\tCtrl-Z', 'Undo the last change')
        editMenu.Append(wx.ID_REDO, '&Redo\tCtrl-Y', 'Redo the last change')
        editMenu.AppendSeparator()
        editMenu.Append(wx.ID_CUT, 'Cu&t\tCtrl-X',
                        'Cut the selected item from the map')
        editMenu.Append(wx.ID_COPY, '&Copy\tCtrl-C',
                        'Copy the selected item from the map')
        editMenu.Append(wx.ID_PASTE, '&Paste\tCtrl-V',
                        'Paste the last cut or copied item onto the map')
        editMenu.AppendSeparator()
        editMenu.Append(wx.ID_PREFERENCES, 'Preferences\tCtrl-P',
                        'Edit the behavior of RatBZEdit')
        # Add the edit menu to the menu bar
        menuBar.Append(editMenu, '&Edit')
        
        # Add menu:
        addMenu = wx.Menu()
        addMenu.Append(ID_Box, '&Box', 'Add a box to the map')
        addMenu.Append(ID_Pyramid, '&Pyramid', 'Add a pyramid to the map')
        addMenu.Append(ID_Base, 'B&ase', 'Add a base to the map')
        # Add the add menu to the menu bar
        menuBar.Append(addMenu, '&Add')
        
        # Help menu:
        helpMenu = wx.Menu()
        helpMenu.Append(wx.ID_ABOUT, '&About', 'About RatBZEdit')
        # Add the help menu to the menu bar
        menuBar.Append(helpMenu, '&Help')
        
        # Set the menu bar:
        self.SetMenuBar(menuBar)
        
        # Bindings for the menu:
        # File:
        self.Bind(wx.EVT_MENU, self.quit, id=wx.ID_EXIT)
        # Edit:
        # Add:
        # Help:
        self.Bind(wx.EVT_MENU, self.menu_about, id=wx.ID_ABOUT)
        
        # Tool bar:
        tb = self.CreateToolBar(wx.TB_HORIZONTAL | wx.NO_BORDER | wx.TB_FLAT)
        
        iconSize = (24, 24)
        save_icon = wx.ArtProvider.GetBitmap(wx.ART_FILE_SAVE, wx.ART_TOOLBAR, iconSize)
        
        tb.SetToolBitmapSize(iconSize)
        
        tb.AddLabelTool(wx.ID_SAVE, 'Save', save_icon, shortHelp='Save',
                        longHelp='Save the current .bzw file')
        
        tb.Realize()
        
        # Main Splitter:
        self.ms = MainSplitter(self, -1)
        sty = wx.SUNKEN_BORDER
        
        p1 = wx.Window(self.ms, style=sty)
        p2 = wx.Window(self.ms, style=sty)
        text = wx.StaticText(p2, -1, 'Map in a GLCanvas', (10, 10))
        self.ms.SplitVertically(p1, p2, 250)
        
        # Status bar:
        self.sb = StatusBar(self)
        self.SetStatusBar(self.sb)
        
        # Set the icon:
        self.SetIcon(wx.Icon('images/window_icon.png', wx.BITMAP_TYPE_PNG, 16,
                             16))
    
    def quit(self, evt):
        '''Event handler to close the program'''
        exit()
    
    def menu_about(self, evt):
        '''Event handler for the "About" item in the "Help" menu'''
        license = open('license.txt')
        info = wx.AboutDialogInfo()
        info.Name = 'RatBZEdit'
        info.Version = '0.0.1'
        info.Copyright = 'Copyright \xc2\xa9 2008 Clayton G. Hobbs'
        info.Description = wordwrap(
            'RatBZEdit is a BZFlag map editor written '
            'with Python and wxPython.\n\n'
            'This program is free software: you can redistribute it and/or '  
            'modify it under the terms of the GNU General Public License as '
            'published by the Free Software Foundation, either version 3 of '
            'the License, or (at your option) any later version.\n\n'
            'This program is distributed in the hope that it will be useful, '
            'but WITHOUT ANY WARRANTY; without even the implied warranty of '
            'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the '
            'GNU General Public License for more details.', 350, wx.ClientDC(self))
        info.Developers = ['Clay Hobbs']
        info.Artists = ['Georgia Hobbs']
        info.License = license.read()
        
        wx.AboutBox(info)
        
        license.close()

#-----{ Run }-----------------------------------------------------------------

if __name__ == '__main__':
    app = wx.App()
    program = SplashScreen()
    
    program.Show()
    app.MainLoop()
